Package: QUBIC
Type: Package
Title: An R Package for Qualitative Biclustering in Support of Gene
        Co-Expression Analyses
Description: The core function of this R package is to provide the
    implementation of the well-cited and well-reviewed QUBIC algorithm, aiming to
    deliver an effective and efficient biclustering capability. This package also
    includes the following related functions: (i) a qualitative representation
    of the input gene expression data, through a well-designed discretization way
    considering the underlying data property, which can be directly used in other
    biclustering programs; (ii) visualization of identified biclusters using heatmap
    in support of overall expression pattern analysis; (iii) bicluster-based
    co-expression network elucidation and visualization, where different correlation
    coefficient scores between a pair of genes are provided; and (iv) a generalize
    output format of biclusters and corresponding network can be freely downloaded
    so that a user can easily do following comprehensive functional enrichment
    analysis (e.g. DAVID) and advanced network visualization (e.g. Cytoscape).
Authors@R: c(person("Yu", "Zhang", role = c("aut", "cre"), email = "zy26@jlu.edu.cn"),
             person("Qin", "Ma", role = "aut", email = "qin.ma@sdstate.edu"))
VignetteBuilder: knitr
biocViews: StatisticalMethod, Microarray, DifferentialExpression,
        MultipleComparison, Clustering, Visualization, GeneExpression,
        Network
Version: 1.41.0
License: CC BY-NC-ND 4.0 + file LICENSE
Encoding: UTF-8
Depends: R (>= 4.5.0)
Imports: Rcpp (>= 0.11.0), methods, Matrix
LinkingTo: Rcpp, RcppArmadillo
Suggests: QUBICdata, qgraph, fields, knitr, rmarkdown
SystemRequirements: C++11, Rtools (>= 3.1)
Enhances: RColorBrewer
URL: https://github.com/zy26/QUBIC
BugReports: https://github.com/zy26/QUBIC/issues
git_url: https://git.bioconductor.org/packages/QUBIC
git_branch: devel
git_last_commit: 98c3be3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:31:56 UTC; biocbuild
Author: Yu Zhang [aut, cre],
  Qin Ma [aut]
Maintainer: Yu Zhang <zy26@jlu.edu.cn>
