useDynLib(QUBIC)
export(BCQU)
export(BCQUD)
export(qubiclust)
export(qubiclust_d)
export(qudiscretize)
export(qunetwork)
export(qunet2xml)
export(quheatmap)
export(showinfo)
importFrom(Rcpp, evalCpp)
S3method(summary,QUBICBiclustResult)
